/* -*-	Mode:C++ -*- */

/*
 * blob.hh
 * Copyright (C) 1999 by John Heidemann
 * $Id: blob.hh,v 1.14 1999/08/15 00:14:12 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */


#ifndef lavaps_blob_h
#define lavaps_blob_h


#include <list>


class process_view;

class blob {
private:
	blob(const blob& tb);
	blob& operator=(const blob& tb);

protected:
	process_view *pv_;

	// blob characteristics.
	int dark_percentage_;
	int dark_offset_;

	double H_, S_, B_;
	int size_;

	static bool next_right_tending_;
		// When rendering blobs with only one column,
		// you may need to fake a 3rd point.
		// It's either right or left per blob,
		// and they're about 50% one way or the other.
	bool right_tending_;

	int x_step_;
	int x_;
	int *y_lows_, *y_highs_;
	int num_;
	int alloced_num_;

	int sign_;  // travel direction
	int reached_limit_;

	typedef list<blob*> blob_redraw_queue_t;
	static blob_redraw_queue_t redraw_queue_;
	blob_redraw_queue_t::iterator redraw_place_;
	void schedule_redraw() { if (redraw_place_ != redraw_queue_.end()) return; redraw_queue_.push_front(this); redraw_place_ = redraw_queue_.begin(); }
	friend class flush_drawing_f;
	static void flush_drawings();

	virtual void redraw() = 0;

	void resize_int_array(int **iap, int on, int nn);
	void adjust_num(int new_num);
	inline bool valid_num(int n) { return n >= 0 && n < num_; }

	static const int DELTA_LIMIT = 5;
	enum blob_op { GROW_OP, MOVE_OP, SHRINK_OP };

	blob *superior();

	int partial_op(blob_op op, int delta);
	virtual void op(blob_op op, int delta);

	void shift_to_right();
	void shift_to_left();
	void widen_on_right();
	void widen_on_left();
	bool narrow_on_right();
	bool narrow_on_left();

	int advance_rank(blob_op op, int num, int delta);
	int find_rank_midpoint(int num);
	int ranks_overlap(int a, int b);

	void reverse_direction();
public:
	blob(process_view *pv);
	virtual ~blob();

	static const int X_MAX = 100, Y_MAX = 200, INITIAL_SLOP = 10;

	static blob *create_real_blob(process_view *pv); // instantiate a back-end blob

	void grow(int delta) { if (delta < 0) op(SHRINK_OP, -delta); else op(GROW_OP, delta); };
	void move(int delta) { op(MOVE_OP, delta); };
	void print();

	virtual void set_hsb(double h, double s, double b);
	virtual void set_size(int size);
	virtual void set_position(int x, int y);
	virtual void set_dark_percentage(int pct, int offset = -1);
	void reordered() { schedule_redraw(); };

	void info_to_buf(char *buf, int len);
};

#endif /* lavaps_blob_h */

