
#
# lava.tcl
# Copyright (C) 1998-1999 by John Heidemann
# $Id: lava.tcl,v 1.21 1999/08/20 17:41:06 johnh Exp $
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

proc init_canvas {} {
	canvas .c -width 100 -height 200 -bg black
	pack .c

	#
	# canvas support
	#
	.c bind p <Any-ButtonPress-1> "info_click_down %x %y %X %Y"
	bind .c <Any-ButtonRelease-1> "info_click_up"
	bind .c <Any-B1-Motion> "info_click_down %x %y %X %Y"
	bind .c <Any-ButtonPress-3> "tk_popup .menu %X %Y 0"
	bind .c <Any-ButtonPress-2> "tk_popup .menu %X %Y 0"
}

proc init_help {} {
	global help
	set help(about) "<big>Lava PS [lava_version]</big>

Copyright (C) 1999 by John Heidemann
All rights reserved.
Comments to <computer><johnh@isi.edu></computer>, latest version at <computer><http://www.isi.edu/~johnh/SOFTWARE/LAVAPS/></computer>

Lava PS is an interactive process-tracking program like ``top'', but with a much different attitude.  Rather than presenting lots of specific info in digital form, it tries to present certain important information in a graphical analog form.  The idea is that you can run it in the background and get a rough idea of what's happening to your system without devoting much concentration to the task.

Lava PS was inspired by Mark Weiser's idea of calm computing.  For details, see:

Mark Weiser and John Seely Brown.  The Coming Age of Calm Technology, Revised version of Weiser & Brown. ``Designing Calm Technology'', PowerGrid Journal, v 1.01, http://powergrid.electriciti.com/1.01 (July 1996). October, 1996. <http://www.ubiq.com/hypertext/weiser/acmfuture2endnote.htm>.

(This program dedicated to the memory of M.W.--I hope you would have thought it a good hack.)
"
}

#
# menu
#
proc init_menu {} {
	global menu rdb
	set m [menu .menu -tearoff 0]

	set mw .menu.who
	menu $mw -tearoff 0
	$mw add radiobutton -label "Me" -variable menu(who) -value me -command "menu_set who {my jobs\n}"
	$mw add radiobutton -label "Everyone" -variable menu(who) -value everyone -command "menu_set who {all jobs\n}"
	set_rdb_default who me {me everyone}
	set menu(who) $rdb(who)
	menu_set who
	$m add cascade -label "Who" -menu $mw

	set mw .menu.what
	menu $mw -tearoff 0
	$mw add radiobutton -label "Virtual Memory" -variable menu(mem) -value virtual -command "menu_set mem {virtual mem\n}"
	$mw add radiobutton -label "Physical Memory" -variable menu(mem) -value physical -command "menu_set mem {physical mem\n}"
	$mw add radiobutton -label "Both" -variable menu(mem) -value both -command "menu_set mem {both mem\n}"
	set_rdb_default what virtual {virtual physical both}
	set menu(mem) $rdb(what)
	menu_set mem
	$m add cascade -label "What" -menu $mw

	$m add command -label "About..." -command "menuHelp about"
	$m add separator

	$m add command -label "Quit" -command "exit 0"
}

proc menu_set {what {feedback ""}} {
	global menu
	lava_menu $what $menu($what)
	if {$feedback != ""} {
		splash_refresh $feedback
	}
}

proc debugging {} {
	global debugging
	if {![info exists debugging]} {
		if {[info exists env(LOGNAME)] && $env(LOGNAME) == "johnh"} {
			set debugging 1
		} else {
			set debugging 0
		}
	}
	return $debugging
}

#
# info boxes
#
proc init_info {} {
	toplevel .info 
	# try and coerce fvwm into doing the right thing
	wm transient .info .
	wm group .
	wm withdraw .info
	wm iconname .info ""
	# xxx: label height should be equal to text height (and dynamic)
	global info_default_wh
	set info_default_wh {24 6}
	if {[debugging]} {
		set info_default_wh {24 7}
	}
	label .info.color -height 1 -width [lindex $info_default_wh 0] -text " "
	text .info.text -width [lindex $info_default_wh 0] -height [lindex $info_default_wh 1]
	pack .info.color
	pack .info.text
	# pack append .info.color "bottom fillx" .info.text "bottom"
	global info_last_x info_last_y info_last_id
	set info_last_x -1
	set info_last_y -1
	set info_last_id -1
}

proc info_click_down {x y rx ry} {
	global info_last_x info_last_y info_last_id

	set id [.c find closest $x $y]

	if {$info_last_id == $id && $info_last_x == $x && $info_last_y == $y} {
		return
	}
	if {$info_last_id == $id} {
		# move existing window
		wm geometry .info "+$rx+$ry"
		return
	}
	global splash_id
	if {$id == $splash_id} {
		return
	}
	set info_last_x $x
	set info_last_y $y
	set info_last_id $id

	.info.color config -background [lindex [.c itemconfigure $id -fill] 4]

	global env
	if {[debugging]} {
		set text "id: $id\n[lava_id_to_info $id]"
	} else {
		set text [lava_id_to_info $id]
	}
	# xxx: should compute width/height
	global info_default_wh
	.info.text delete 0.0 end
	.info.text insert 0.0 $text
	.info.text config -width [lindex $info_default_wh 0] -height [lindex $info_default_wh 1]
	wm geometry .info "+$rx+$ry"
	wm deiconify .info
	# xxx: should detect if window goes off edge and reposition it automatically
}
proc info_click_up {} {
	global info_last_id
	set info_last_id -1
	wm withdraw .info
}

#
# intro splash
#
proc splash_create {text} {
	global splash_id splash_gray
	set splash_id [.c create text 50 50 -anchor center -fill white -text $text -font {-family Helvetica -weight bold}]
	set splash_gray 255
	after 4000 splash_fade
}
proc splash_refresh {text} {
	global splash_id splash_gray
	if {$splash_id == "dead"} {
		splash_create $text
	} else {
		# reset it's timer and add the text
		set splash_gray 255
		.c insert $splash_id end $text
		# xxx: the user can overflow the text on the screen.
		# Fortunately, this is just a cosmetic problem.
	}
}
proc splash_fade {} {
	global splash_id splash_gray
	incr splash_gray -6
	if {$splash_gray <= 0} {
		.c delete $splash_id
		set splash_id dead
	} else {
		set g [format "%02x" $splash_gray]
		.c itemconf $splash_id -fill "#$g$g$g"
		.c itemconf raise $splash_id
		after 1000 splash_fade
	}
}
proc init_splash {} {
	global splash_id
	set splash_id dead
}

#
# controls:
#
proc make_scale {tag} {
	# scale
}
proc make_scales {} {
	make_scale mem
}

#
# debugging input
#
if {0} {
	fconfigure stdin -blocking false -buffering line
	fileevent stdin readable lava_debug_callback
	proc lava_debug_callback {} {
		gets stdin line
		puts "lava_debug_callback: $line"
		eval $line
	}
	# proc m {} { lava_move }
	# proc g {} { lava_grow }
	# proc p {} { lava_print }
	# proc r {} { lava_reverse }
}

#
# monitoring work
#
proc lava_ticker {} {
	lava_tick

	# splash exists is only in tcl, raise it here
	global splash_id
	if {$splash_id != "dead"} {
		.c raise $splash_id
	}

	global lava_tick_interval
	after $lava_tick_interval lava_ticker
}
proc init_ticker {} {
	global lava_tick_interval
	set lava_tick_interval 2000
	# On my 266MHz Pentium, 2000 == ~6% CPU usage to a remote display.
	# must defer first check to set up canvas
	after 10 lava_ticker
}


#
# main
#
proc main {} {
	wm iconname . "lavaps"
	wm title . "lavaps"

	init_splash
	init_resources
	init_canvas
	init_help
	init_menu
	init_info

	splash_refresh "buttons:\nleft: info\nright: menu\n"

	init_ticker
}
